/** TileMap2DScene.cpp
 *
 * See header file for details.
 */

#include "TileMap2DScene.h"

#include "TileMap2D.h"
#include "TrackingCamera2D.h"

const static TrackingCamera2D defaultCamera;

TileMap2DScene::TileMap2DScene() {
}

TileMap2DScene::TileMap2DScene(const std::string &fileName) :
	tileMap(std::make_shared<TileMap2D>(fileName)) {
}

TileMap2DScene::~TileMap2DScene() {
}

raylib::Vector2 TileMap2DScene::get2DSize() const {
	if(tileMap) {
		return tileMap->getMapSize();
	} else {
		return Scene::get2DSize();
	}
}

std::shared_ptr<Scene> TileMap2DScene::update() {
	float elapsedTime = GetFrameTime();
	if(tileMap) {
		tileMap->updateAnimations(elapsedTime);
	}
	
	// ##### FIXME! ###### Handle collisions?
	
	return Scene::update();
}

void TileMap2DScene::draw() {
	ClearBackground(RAYWHITE);
	
	if(camera) {
		camera->BeginMode();
	}
	if(tileMap) {
		auto &currCamera = camera ? *camera : defaultCamera;
		tileMap->draw(currCamera, WHITE);
	}
	
	drawActors();
	
	if(camera) {
		camera->EndMode();
	}
}
